﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;

namespace HeuristicLab.Problems.BinPacking.Dimensions {
  [Item("Three Dimensional Packing", "Represents a packing-position associated with a three dimensional packing-problem.")]
  [StorableClass]
  public class ThreeDimensionalPacking: PackingDimensions {
    public int X { get; set; }
    public int Y { get; set; }
    public int Z { get; set; }

    [StorableConstructor]
    protected ThreeDimensionalPacking(bool deserializing) : base(deserializing) { }
    protected ThreeDimensionalPacking(ThreeDimensionalPacking original, Cloner cloner)
      : base(original, cloner) {
      X = original.X;
      Y = original.Y;
      Z = original.Z;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ThreeDimensionalPacking(this, cloner);
    }

    public ThreeDimensionalPacking(int assignedBin, int x, int y, int z) : this(assignedBin, x, y, z, false) {}
    public ThreeDimensionalPacking(int assignedBin, int x, int y, int z, bool rotated)
      : base(assignedBin, rotated) {
      this.X = x;
      this.Y = y;
      this.Z = z;
    }
    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append("[");
      sb.Append("AssignedBin: ");
      sb.Append(AssignedBin);
      sb.Append("; (");
      sb.Append(X + ",");
      sb.Append(Y + ",");
      sb.Append(Z);
      sb.Append(")");
      sb.Append("]");
      return sb.ToString();
    }


    public override bool Equals(object obj) {
      var tdp = obj as ThreeDimensionalPacking;
      if (tdp != null)
        return (tdp.X == this.X && tdp.Y == this.Y && tdp.Z == this.Z);
      else return false;
    }

    public override int GetHashCode() {
      return base.GetHashCode() + 13 * X + 17 * Y + 23 * Z;
    }

    public static ThreeDimensionalPacking MoveLeft(ThreeDimensionalPacking original) {
      return new ThreeDimensionalPacking(original.AssignedBin, original.X - 1, original.Y, original.Z, original.Rotated);
    }
    public static ThreeDimensionalPacking MoveDown(ThreeDimensionalPacking original) {
      return new ThreeDimensionalPacking(original.AssignedBin, original.X, original.Y - 1, original.Z, original.Rotated);
    }
    public static ThreeDimensionalPacking MoveBack(ThreeDimensionalPacking original) {
      return new ThreeDimensionalPacking(original.AssignedBin, original.X, original.Y, original.Z - 1, original.Rotated);
    }

    public static ThreeDimensionalPacking MoveRight(ThreeDimensionalPacking original) {
      return new ThreeDimensionalPacking(original.AssignedBin, original.X + 1, original.Y, original.Z, original.Rotated);
    }
    public static ThreeDimensionalPacking MoveUp(ThreeDimensionalPacking original) {
      return new ThreeDimensionalPacking(original.AssignedBin, original.X, original.Y + 1, original.Z, original.Rotated);
    }
    public static ThreeDimensionalPacking MoveFront(ThreeDimensionalPacking original) {
      return new ThreeDimensionalPacking(original.AssignedBin, original.X, original.Y, original.Z + 1, original.Rotated);
    }
  }
}
